
/*===============================================*\
	Samedi 7 novembre 1998

	Arash HABIBI

	Medium.c

	Ouvre deux sockets, et ecrit dans l'une 
	tout ce qu'il peut lire sur l'autre et 
	reciproquement. Pourra eventuellement 
	y ajouter des erreurs. 
	On suppose que les sockets distantes 
	s'appellent sock_emit et sock_rec et 
	qu'elles sont dans le meme repertoire. 

\*================================================*/

#include "define.h"
#include "SocketToolbox.h"
#include "MediumToolbox.h"
#include "Erreur.h"
#include "Attente.h"

int   terminal=-1, 
  sock_medium1=-1, 
  sock_medium2=-1;


void ExitPropre()
/*================================================*\
  Pour terminer le processus en fermant fichiers, 
  terminaux et sockets. 
\*================================================*/
{ 
  close(sock_medium2);
  close(sock_medium1);
  close(terminal);
  remove("sock_medium1");
  remove("sock_medium2"); 

  printf("\n\nMEDIUM : Ciao !\n\n");

  exit(0);
}



int Echange(int s_rec, int s_em, struct sockaddr_un adr_em, int type, float taux)
/*===============================================*\
  D'apres select nous savons qu'il y a 
  quelque chose a lire sur s_rec. 
  Le but de cette fonction est de lire
  ce quelque chose et de l'envoyer sur 
  s_em. 
  La fonction rend -1 en cas d'erreur,
  1 si tout va bien, et 0 quand une des
  sockets envoie une trame qui ne contient 
  qu'un seul caractere '.' 
\*===============================================*/
{
    int nb_rec, nb_emis, taille_adresse=110; 
    char trame[MAX_TRAME]; 
    struct sockaddr_un adr_rec; 
    
    nb_rec = recvfrom(s_rec, trame, MAX_TRAME, 0, 
		    (struct sockaddr *) &adr_rec,
		    &taille_adresse); 

    if(nb_rec<0) {perror("read socket"); 
    	return(-1);}
    
    else if(nb_rec>=0)
      { 
      	IntroduitErreur(trame, nb_rec, type, taux);

	nb_emis = sendto(s_em, trame, nb_rec, 0, 
			 (struct sockaddr*) &adr_em, 
			 sizeof(adr_em));
	
	if(nb_emis<0) {perror("sendto"); return(-1);}
	
	else return(1);
      }
}



void main(int argc, char **argv)
/*===============================================*\
  Ouvre et prepare deux sockets unix, 
\*===============================================*/
{
    struct sockaddr_un adr_dist_stat1, adr_dist_stat2;
    char param[100];
    boolean stop=FAUX, crash=FAUX; 
    float taux=0.0;              /* taux d'erreur */
    int type=0;                  /* type d'erreur */
    int taille;
    int commande, nb_arg, res, evenement;  

    if(argc!=1) fprintf(stderr, "\nUsage : medium\n\n");

/*------------ ExitPropre en cas de ctrl C ------------------*/

    EnCasDeCtrl_C(&ExitPropre);

/* ----- creation et attachement des sockets --------------*/
/* ----- preparation de l'adresse de la socket distante -- */

    sock_medium1 = GetAndBindSocket("sock_medium1");
    sock_medium2 = GetAndBindSocket("sock_medium2");

    adr_dist_stat1 = PrepareAdresse("sock_station1"); 
    adr_dist_stat2 = PrepareAdresse("sock_station2"); 
    
/* ----- Le terminal de controle --------------------------*/

  terminal = OpenTerminal();
  PrintPrompt(terminal, "MEDIUM :) ");

/* ----- Echanges de donnees ------------------------------*/  
    
    while((!stop)&&(!crash))
	{
      evenement = Attendre(terminal, 
			   sock_medium1,
			   sock_medium2, 
			   0.0);

      if(evenement==-1) crash = VRAI; 
      else
	{
/*----------- le depouillement des votes ------- */
/*-------- une commande venant du terminal ------*/

	  if(((evenement==YaALireSur1__)||(evenement==YaALireSur12_))||
	     ((evenement==YaALireSur1_3)||(evenement==YaALireSur123)))
	    {
	      commande = LectCommande(terminal, param, &nb_arg);

	      switch(commande)
		{
		case TAUX  : ChangeTaux(nb_arg, param, &taux); break;
		case TYPE  : ChangeType(nb_arg, param, &type); break ;
		case SHELL : ShellCommand(param); break ;
		case HELP  : Help(); break; 
		case QUIT  : stop=VRAI; break; 
		default    : break; 
		}
	      if(stop==FAUX)
	      	PrintPrompt(terminal, "MEDIUM :) ");
	    }

/*----- le message vient de station 1 ----*/

	  if(((evenement==YaALireSur_2_)||(evenement==YaALireSur12_))||
	     ((evenement==YaALireSur_23)||(evenement==YaALireSur123)))
		{
		  if(Echange(sock_medium1, sock_medium2, 
				adr_dist_stat2, type, taux)<0) 
		  crash=VRAI; 
		}

/*-----le message vient de station 2------*/
	    
	  if(((evenement==YaALireSur__3)||(evenement==YaALireSur1_3))||
	     ((evenement==YaALireSur_23)||(evenement==YaALireSur123)))
		{	
		  if(Echange(sock_medium2, sock_medium1, 
				adr_dist_stat1, type, taux)<0) 
			crash=VRAI;
		}
	}
     }
    if(crash)
	printf("Echange interrompu");
    else if(stop)
	printf("Echange termine");

    ExitPropre();
}
