
/*================================================*\

	Mardi 24 novembre 1998

	Arash HABIBI

	Attente.c

	A chaque fois qu'il faut attendre un 
	evenement c'est ici qu'il faut s'adresser. 

\*================================================*/


#include "Attente.h"
#include "Timer.h"


int Max(int x, int y, int z, int t)
/*================================================*\
  Rend le plus grand des trois entiers. 
\*================================================*/
{
  int max=0; 

  max=x; 

  if(y>max) max=y; 
  if(z>max) max=z; 
  if(t>max) max=t;
  
  return(max); 
}


int Attendre(int fd1, int fd2, int fd3, int fd4, float timeout)
/*================================================*\
  Attend que l'un des deux descripteurs fd1, fd2, fd3
  ou fd4 ou les quatre deviennent lisibles ou alors que 
  le temps "timeout" (en secondes) soit ecoule. fd1 
  fd2, fd3 et fd4 peuvent etre des descripteurs de fichier, 
  de terminal, de socket ou de tube. 
  
  Si timeout=0 alors le temporisateur ne sera pas 	 
  arme. De meme si l'un des descripteurs est negatif
  (par exemple RIEN) alors l'argument sera ignore. 
  En particulier Attendre(RIEN, RIEN, RIEN, RIEN, 3.0); 
  ne fait qu'attendre 3 secondes avant de rendre la main. 

  La fonction rend YaALireSur1___ (i.e. 1) si fd1 est seul lisible
                   YaALireSur_2__ (i.e. 2) si fd2 est seul lisible
		   YaALireSur12__ (i.e. 3) si fd1 et fd2 sont lisibles
		   YaALireSur__3_ (i.e. 4) si fd3 est seul lisible
		   YaALireSur1_3_ (i.e. 5) si fd1 et fd3 sont lisibles
		   YaALireSur_23_ (i.e. 6) si fd2 et fd3 sont lisibles
		   YaALireSur123_ (i.e. 7) si fd1, fd2 et fd3 sont lisibles
		   YaALireSur___4 (i.e. 8) si fd4 est seul lisible 
		   YaALireSur1__4 (i.e. 4) si fd1 et fd4 sont lisibles
                   YaALireSur_2_4 (i.e. 10) si fd2 et fd4 sont lisibles
		   YaALireSur12_4 (i.e. 11) si fd1, fd2 et fd4 sont lisibles
		   YaALireSur__34 (i.e. 12) si si fd3 et fd4 sont lisibles
		   YaALireSur1_34 (i.e. 13) si fd1, fd3 et fd4 sont lisibles
		   YaALireSur_234 (i.e. 14) si fd2, fd3 et fd4 sont lisibles
		   YaALireSur1234 (i.e. 15) Les quatre sont lisibles.
		   TIMEOUT        (i.e. 0) si aucun n'est lisible avant
		     une duree de timeout. 

  IMPORTANT : on travaille seulement en lecture.

\*================================================*/
{
    fd_set lecture, ecriture, except;
    int nbfds, nb_events, res; 
    struct timeval timer;

    /*---- Preparation du timer -------- */

    timer.tv_sec  = (int)floor(timeout); 
    timer.tv_usec = (int)floor(1000000*(timeout-(int)floor(timeout)));

    /*---- Preparation des ensembles -------- */
    /*--------- de descripteurs ------------- */
    
    nbfds = Max(fd1, fd2, fd3, fd4)+1; 
    if(nbfds<=0) nbfds=1; 
    
    FD_ZERO(&lecture); 
    FD_ZERO(&ecriture); 
    FD_ZERO(&except); 
    if(fd1>=0) FD_SET(fd1, &lecture); 
    if(fd2>=0) FD_SET(fd2, &lecture); 
    if(fd3>=0) FD_SET(fd3, &lecture); 
    if(fd4>=0) FD_SET(fd4, &lecture); 
    
    /*------- attente d'un evenment --------- */
    
    do
      {
	if(timeout==0)
	  nb_events = select (nbfds, &lecture, &ecriture, 
			      &except, NULL);
	else
	  nb_events = select (nbfds, &lecture, &ecriture, 
			      &except, &timer);	
	
	if((nb_events==-1)&&(errno!=EINTR))
	  {  perror("select"); return(CRASH);}

	/* Si le select est interrompu par le signal    */
        /* SIGALARM utilise pour les timeout, il faut   */
        /* reprendre l'attente comme si rien ne s'etait */
        /* passe. C'est le sens de la ligne :           */
        /* "while(errno!=EINTR)" en bas du fichier.     */
	
     /*------- le depouillement des votes ---- */
	
	else
	  {
	    res = 0; 
	    
	    if((fd1>=0)&&(FD_ISSET(fd1, &lecture)))
	      res+=1; 
	    
	    if((fd2>=0)&&(FD_ISSET(fd2, &lecture)))
	      res+=2; 
	    
	    if((fd3>=0)&&(FD_ISSET(fd3, &lecture)))
	      res+=4; 
	      
	    if((fd4>=0)&&(FD_ISSET(fd4, &lecture)))
	      res+=8; 
	    
	    /* implicitement si aucune des trois conditions n'est */
	    /* verifiee, alors c'est qu'on est en prensence d'un  */
	    /* magnifique timeout : res = 0                       */
	  }
      } 
    while((nb_events==-1)&&(errno==EINTR));

    return(res);
}



