
/*================================================*\

	Lundi 23 novembre 1998

	Arash HABIBI

	Physique.c

      Tout ce qui concerne la couche physique

\*================================================*/


#include "Physique.h"


void InitCouchePhys(CouchePhys_t *CchPhys, char *loc, char *dist, int data_pl, int data_lp, int disc_pl, int disc_lp)
/*================================================*\
 Initialise les quelques variables (essentiellement
 les descripteurs de tube entre la couche physique et
 la couche liaison, et ensuite commence la boucle de 
 la couche physique qui attend que les evenements 
 aient lieu. 
\*================================================*/
{
  CchPhys->Data_PL =  data_pl; 
  CchPhys->Data_LP =  data_lp; 
  CchPhys->Dcnx_PL =  disc_pl;
  CchPhys->Dcnx_LP =  disc_lp;

  strcpy(CchPhys->LocalSocketName, loc);
  strcpy(CchPhys->DistantSocketName, dist);

  SetLocAdrSock(loc, CchPhys);
  SetDstAdrSock(dist, CchPhys); 

  Phys_Loop(CchPhys);
}


void FreeCouchePhys(CouchePhys_t *CchPhys)
/*================================================*\
 ferme la socket et efface les fichiers 
 temporaires qui ont ete crees. 
\*================================================*/
{
  printf("Couche physique deconnectee.\n");
  close(CchPhys->desc_sock);
  remove(CchPhys->LocalSocketName);
}



void SetLocAdrSock(char *ref, CouchePhys_t *CchPhys)
/*================================================*\
 l'equivalent de GetAndBindSocket. Ouvre une 
 socket et l'attache a l'adresse ref.
\*================================================*/
{
  CchPhys->desc_sock = socket(AF_UNIX, SOCK_DGRAM, 0);
    
    if(CchPhys->desc_sock == -1) {perror("socket"); exit(1);}
    
    /* ----- preparation de l'adresse locale ----- */
    
    CchPhys->adresse_loc.sun_family = AF_UNIX; 
    strcpy(CchPhys->adresse_loc.sun_path, ref);

    /*----- et attachement de la socket a   ----- */
    /*----- l'adresse ref par bind          ----- */
	
    if(bind(CchPhys->desc_sock, 
	    (struct sockaddr*) &(CchPhys->adresse_loc),
	       sizeof(CchPhys->adresse_loc))==-1)
      {perror(ref); exit(1);}
}




void SetDstAdrSock(char *ref, CouchePhys_t *CchPhys)
/*================================================*\
 Caclule simplement l'adresse de la socket 
 distante
\*================================================*/
{
  CchPhys->adresse_dist.sun_family = AF_UNIX; 
  strcpy(CchPhys->adresse_dist.sun_path, ref);    
}


int Phys_Disconnect(CouchePhys_t *CchPhys)
/*================================================*\
 Cette fonction est appelee quand la couche 
 physique a recu un message de la part de 
 la couche liaison sur le SAP Deconnexion
 Dans ce cas, il ne peut s'agir que d'une 
 demande de deconnexion. C'est pourquoi on 
 envoie une trame vide qui ne contient que 
 le caractere '.'
 On retourne 1 pour dire que tout s'est 
 bien passe. 
\*================================================*/
{
  if(sendto(CchPhys->desc_sock, ".", 1, 0, 
	    (struct sockaddr*) &(CchPhys->adresse_dist), 
	    TAILLE_ADRESSE)==-1)
    {perror("Phys_Disconnect sendto"); return(-1);}
  else
    return(1);
}


int Phys_SendData(CouchePhys_t *CchPhys)
/*================================================*\
 On vient de recevoir des donnees de la part
 de la couche liaison sur le SAP Data. Donc 
 il ne reste plus qu'a l'envoyer sur la 
 socket. On retourne 1 pour dire que tout 
 s'est bien passe, et -1 pour 
 dire qu'il y a eu un probleme. 
\*================================================*/
{
  int taille_trame, taille_adresse=TAILLE_ADRESSE; 
  char trame[TAILLE_TRAME];

  if((taille_trame = read(CchPhys->Data_LP, 
			  trame, 
			  TAILLE_TRAME))<=0)
    {perror("Phys_SendData read"); return(-1);}

  else if(sendto(CchPhys->desc_sock, trame, taille_trame, 0, 
	    (struct sockaddr*) &(CchPhys->adresse_dist), 
	    TAILLE_ADRESSE)==-1)
    {perror("Phys_SendData sendto"); return(-1);}

  else
    return(0);
}



int Phys_ReceiveData(CouchePhys_t *CchPhys)
/*================================================*\
 On vient de recevoir des donnees sur la socket
 il ne reste plus qu'a les envoyer a la 
 couche liaison sur le SAP Data ou 
 le SAP Disconnect. 
\*================================================*/
{
  int taille_trame, taille_adresse=TAILLE_ADRESSE; 
  char trame[TAILLE_TRAME], disconnect[1];

  if((taille_trame = recvfrom(CchPhys->desc_sock, 
			      trame, TAILLE_TRAME, 0, 
			  (struct sockaddr *) &(CchPhys->adresse_dist),
			  &taille_adresse))<=0)
    {perror("Phys : recvfrom"); return(-1);}

  else if((taille_trame==1)&&(trame[0]=='.'))
    {
      /*--- On a recu le signal de deconnexion qu'on delivre ------ */
      /*--- comme un disconnect indication a la couche superieure - */

      if(write(CchPhys->Dcnx_PL, disconnect, 1)==-1)
	{perror("Phys_ReceiveData : write1"); return(-1);}
      else 
	return(0);
    }

      /*--- On a recu de l'information ordinaire qu'on ---- */
      /*--- delivre comme data a la couche superieure ----- */

  else if(taille_trame>0)
    if(write(CchPhys->Data_PL, trame, taille_trame)==-1)
      {perror("Phys_ReceiveData : write2"); return(-1);}
    else
      return(1);
}



void Phys_Loop(CouchePhys_t *CchPhys)
/*================================================*\
 la grande boucle de la couche physique qui 
 attend les evenements qui peuvent se produire : 
 il y a trois evenements possible : 
 - arrive d'une trame sur la socket
 - arrive d'un message de la part de la couche liaison
 - arrive d'une demande de deconnexion venant de 
   la couche liaison.
\*================================================*/
{
    fd_set lecture, ecriture, except;
    int nbfds, nb_events, res; 
    boolean crash=FAUX, stop=FAUX;

    nbfds = Max(CchPhys->desc_sock, CchPhys->Data_LP, 
		CchPhys->Dcnx_LP) + 1;

    printf("La couche physique commence a boucler.\n");
    
    while((!stop)&&(!crash))
	{
            /*---- Preparation des ensembles -------- */
            /*--------- de descripteurs ------------- */
	    
	    FD_ZERO(&lecture); 
	    FD_ZERO(&ecriture); 
	    FD_ZERO(&except); 
	    FD_SET(CchPhys->desc_sock, &lecture); 
	    FD_SET(CchPhys->Data_LP, &lecture); 
	    FD_SET(CchPhys->Dcnx_LP, &lecture); 

	    
            /*------- attente d'un evenment --------- */

	    nb_events = select (nbfds, &lecture, &ecriture, 
				&except, NULL);
	    if(nb_events==-1) {perror("select"); crash=VRAI;} 
	    
	    /*------- le depouillement des votes ---- */

	    else
	      {
		/*---------- nouvelle venant de la socket ---*/

		if(FD_ISSET(CchPhys->desc_sock, &lecture))
		  {
		    res = Phys_ReceiveData(CchPhys);
		    if(res<0) crash=VRAI; 
		    else if(res==0) stop=VRAI; 
		    else /* tout va bien */;
		  }

		/*---- donnees venant de la couche reseau ----*/
		
		if(FD_ISSET(CchPhys->Data_LP, &lecture))
		  {
		    if(Phys_SendData(CchPhys)<0)
		      crash=VRAI; 
		  }

		/*---- disconnect request venant de la couche reseau ----*/
		
		else if(FD_ISSET(CchPhys->Dcnx_LP, &lecture))
		  if(Phys_Disconnect(CchPhys)<0) 
		    crash=VRAI; 
		  else
		    stop=VRAI;
	      }
	}
    if(stop==VRAI) printf("Echange termine.\n");
    else /*crash==VRAI*/ printf("Echange interrompu.\n");
}



